@@ -2,7 +2,7 @@
 
 /**
   * SquirrelMail Multilogin Plugin
-  * Copyright (c) 2003-2007 Paul Lesniewski <paul@squirrelmail.org>
+  * Copyright (c) 2003-2010 Paul Lesniewski <paul@squirrelmail.org>
   * Licensed under the GNU GPL. For full terms see the file COPYING.
   *
   * @package plugins
@@ -12,6 +12,25 @@
 
 
 
+/**
+  * Initialize this plugin (load config values)
+  *
+  * @return boolean FALSE if no configuration file could be loaded, TRUE otherwise
+  *
+  */
+function multilogin_init()
+{
+
+   if (!@include_once (SM_PATH . 'config/config_multilogin.php'))
+      if (!@include_once (SM_PATH . 'plugins/multilogin/config.php'))
+         return FALSE;
+
+   return TRUE;
+
+}
+
+
+
 /**
   * Validate that this plugin is configured correctly
   *
@@ -22,33 +41,45 @@
 function multilogin_check_configuration_do()
 {
 
+   // make sure configuration file exists
+   //
+   if (!multilogin_init())
+   {
+      do_err('Multilogin plugin is missing its configuration file; please consult its INSTALL document', FALSE);
+      return TRUE;
+   }
+
+
    // only need to do this pre-1.5.2, as 1.5.2 will make this
    // check for us automatically
    //
    if (!check_sm_version(1, 5, 2))
    {
 
-      // try to find Compatibility, and then that it is v2.0.7+
+      // try to find Compatibility, and then that it is v2.0.12+
       //
       if (!function_exists('check_plugin_version')
-       || !check_plugin_version('compatibility', 2, 0, 7, TRUE))
+       || !check_plugin_version('compatibility', 2, 0, 12, TRUE))
       {
-         do_err('Multilogin plugin requires the Compatibility plugin version 2.0.7+', FALSE);
+         do_err('Multilogin plugin requires the Compatibility plugin version 2.0.12+', FALSE);
          return TRUE;
       }
 
-
-      // make sure configuration file exists
-      //
-      if (!load_config('multilogin', 'config.php', TRUE))
-      {
-         do_err('Multilogin plugin is missing its configuration file; please consult its INSTALL document', FALSE);
-         return TRUE;
-      }
-
-
    }
 
+
+   // check that the needed patch is in place for SM versions < 1.5.2
+   //
+   if (!check_sm_version(1, 5, 2))
+   {
+      if (!check_file_contents(SM_PATH . 'include/validate.php', 'include_once\(SM_PATH \. \'plugins\/multilogin\/functions.php\'\);\s+multilogin_sqoverride_config_do\('))
+      {
+         do_err('Multilogin plugin requires a patch with your version of SquirrelMail, but it has not been applied', FALSE);
+         return TRUE;
+      }
+   }
+
+
    return FALSE;
 
 }
@@ -64,6 +95,7 @@ function multilogin_registerIMAPServer_do($args)
 
    $current_hook_name = get_current_hook_name($args);
 
+//TODO: reevaluate if the following is really true/needed
    // php 4.3 confuses login info when trying 
    // to login without having logged out last time
    //
@@ -83,13 +115,7 @@ function multilogin_registerIMAPServer_do($args)
    sqGetGlobalVar('rememberLoginServer', $rememberLoginServer, SQ_FORM);
 
 
-   // can't use load_config here because compatibility
-   // plugin isn't in the include path prior to this
-   // hook call... but that's OK, as we were already
-   // able to use load_config on the login page
-   //
-   //load_config('multilogin', 'config.php');
-   include_once(SM_PATH . 'plugins/multilogin/config.php');
+   multilogin_init();
 
 
    // somehow, we lose the main SM "key" cookie by doing this
@@ -117,8 +143,17 @@ function multilogin_registerIMAPServer_do($args)
    {
 
       global $config_override;
-      sqsession_unregister('config_override');
-////      $config_override = array();
+      $config_override = array();
+
+      // This wipes anything Vlogin had already added, so I am commenting
+      // it out.  I think this line was originally just trying to be
+      // thorough, but should not be necessary since session is wiped on
+      // login page.  Instead, we actually need to get what is in the
+      // session already.
+      //
+      //sqsession_unregister('config_override');
+      sqGetGlobalVar('config_override', $config_override, SQ_SESSION);
+
 
       foreach ($IMAPSelections[$loginServer] as $setting => $value)
       {
@@ -129,7 +164,12 @@ function multilogin_registerIMAPServer_do($args)
 
       sqsession_register($config_override, 'config_override');
 
-      multilogin_sqoverride_config_do($args);
+
+      // I can't see the need for this on the login_verified hook
+      // so making it conditional...
+      //
+      if (get_current_hook_name($args) == 'login_before')
+         multilogin_sqoverride_config_do($args);
 
    }
    else
@@ -155,7 +195,7 @@ function multilogin_insertIMAPSelection_do()
 
    global $enableIMAPSelection, $IMAPSelections, $rememberServer, $color;
 
-   load_config('multilogin', 'config.php');
+   multilogin_init();
 
 
    sq_change_text_domain('multilogin');
@@ -284,6 +324,56 @@ function multilogin_insertIMAPSelection_do()
 
 
 
+/**
+  * Does a couple things before passing execution
+  * to multilogin_sqoverride_config_do().
+  *
+  */
+function multilogin_pre_sqoverride_config_do($args='')
+{
+
+   global $PHP_SELF;
+
+   // in 1.4.x, we don't want to run on the prefs_backend hook
+   // (that's just for config_override in 1.5.x)
+   // the multilogin SM core patch takes care of this in 1.4.x
+   //
+   if (!check_sm_version(1, 5, 2))
+      return;
+
+
+   // In SM 1.5.2+, the login page tries to test if
+   // the IMAP server supports plain text logins...
+   // but when using this plugin, the main IMAP server
+   // from the SM configuration is not necessarily
+   // the server being used, so we have to trick that
+   // code not to run
+   //
+   else if (strpos($PHP_SELF, '/login.php') !== FALSE)
+   {
+      global $imap_auth_mech;
+      $temp_imap_auth_mech = $imap_auth_mech;
+      $imap_auth_mech = FALSE;
+   }
+
+
+   // The vlogin plugin will already call this on its own,
+   // so if it is in use, we'll let it do that and not do
+   // it ourselves to reduce uneccessary code execution
+   //
+   // (well, the truth is if vlogin's $useSessionBased
+   // is turned off it *won't* call multilogin_sqoverride_config_do,
+   // but if running multilogin, there is no reason NOT
+   // to have $useSessionBased turned on; the vlogin configtest
+   // makes a check for this)
+   //
+   if (!is_plugin_enabled('vlogin'))
+      multilogin_sqoverride_config_do($args);
+
+}
+
+
+
 /**
   * This will look for config overrides in the session,
   * which must be in an array called 'config_override'.
@@ -296,6 +386,7 @@ function multilogin_sqoverride_config_do($args='')
 
    if (sqsession_is_registered('config_override'))
    {
+      // just debugging; nothing to see here...  $backtrace = debug_backtrace(); $b=''; foreach ($backtrace as $x) {$b.=$x['function'] . '('; foreach ($x['args'] as $arg) if (is_array($arg)) $b.='ARRAY[' . implode(', ',$arg) . '], '; else $b .= $arg .', '; $b= substr($b, 0, -2) . '), ';}$b.="\n=======================================================\n"; exec("echo '".date('H:i:s')."  :  ".$_SERVER['SCRIPT_NAME']."  :  ".$b."' >> /tmp/override_config_trace;");
 
       global $plugins, $squirrelmail_plugin_hooks;
       $config_override = array();
